package com.gx.obe.util.element;


import java.beans.IntrospectionException;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Field;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

public class ElementClass<T> {
	
	private Class<T> tClass;
	
	private List<FieldInfo> fieldInfoList;
	
	public static <T> ElementClass<T> of(Class<T> tClass) {
		return new ElementClass<>(tClass);
	}
	
	public ElementClass(Class<T> tClass) {
		this.tClass = tClass;
		this.fieldInfoList = Arrays.stream(tClass.getDeclaredFields()).map(this::createFieldInfo).filter(t -> t != null).collect(Collectors.toList());;
	}
	
	private FieldInfo createFieldInfo(Field field) {
		PropertyDescriptor descriptor = null;
		try {
			descriptor = new PropertyDescriptor(field.getName(), tClass);
		} catch (IntrospectionException e) {
			return null;
		}
		return new FieldInfo(field, descriptor.getWriteMethod());
	}
	
	public T create(Element element) {
		T t;
		try {
			t = tClass.newInstance();
		} catch (InstantiationException | IllegalAccessException e) {
			return null;
		}
		fieldInfoList.forEach(fieldInfo -> fieldInfo.mapValue(t, element));
		return t;
	}
	
	public List<T> createList(Element element) {
		List<?> elements = element.elements();
		return elements.stream().map(t -> create((Element) t)).collect(Collectors.toList());
	}
	
	public T create(String xml) {
		try {
			return create(DocumentHelper.parseText(xml).getRootElement());
		} catch (DocumentException e) {
			throw new RuntimeException(e);
		}
	}
	
	public List<T> createList(String xml) {
		try {
			return createList(DocumentHelper.parseText(xml).getRootElement());
		} catch (DocumentException e) {
			throw new RuntimeException(e);
		}
	}
	
}
