package com.gx.obe.util.utils;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.apache.log4j.Logger;

import com.gx.obe.util.entity.BaseEntity;

/**
 * @ClassName: CommonUtil
 * @Description:
 * @author guoyr
 */
public abstract class BeanUtils {

	private static Logger LOG = Logger.getLogger(BeanUtils.class);
	private static final char[] HEX = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e',
			'f' };

	/**
	 * Returns true if the provided class is a type supported natively (as opposed
	 * to a bean).
	 * 
	 * @param type {@link java.lang.Class} type to be tested
	 */
	public static boolean isNativeType(final Class<?> type) {

		// to return an arbitrary object use Object.class
		return (type == boolean.class || type == Boolean.class || type == byte.class || type == Byte.class
				|| type == short.class || type == Short.class || type == int.class || type == Integer.class
				|| type == long.class || type == Long.class || type == float.class || type == Float.class
				|| type == double.class || type == Double.class || type == char.class || type == Character.class
				|| type == byte[].class || type == Byte[].class || type == char[].class || type == Character[].class
				|| type == String.class || type == BigDecimal.class || type == java.util.Date.class
				|| type == java.sql.Date.class || type == java.sql.Time.class || type == java.sql.Timestamp.class
				|| type == java.io.InputStream.class || type == java.io.Reader.class || type == java.sql.Clob.class
				|| type == java.sql.Blob.class || type == Object.class);
	}

	public static boolean matched(Class<?> nativeType, String className) {
		if (nativeType.getName().equals(className)) {
			return true;
		}
		if ((nativeType == int.class || nativeType == Integer.class)
				&& ("int".equals(className) || "java.lang.Integer".equals(className))) {
			return true;
		}
		if ((nativeType == int.class || nativeType == Integer.class)
				&& ("long".equals(className) || "java.lang.Long".equals(className))) {
			return true;
		}
		if ((nativeType == long.class || nativeType == Long.class)
				&& ("long".equals(className) || "java.lang.Long".equals(className))) {
			return true;
		}
		if ((nativeType == long.class || nativeType == Long.class)
				&& ("int".equals(className) || "java.lang.Long".equals(className))) {
			return true;
		}
		if ((nativeType == double.class || nativeType == Double.class)
				&& ("double".equals(className) || "java.lang.Integer".equals(className))) {
			return true;
		}
		if ((nativeType == float.class || nativeType == Float.class)
				&& ("float".equals(className) || "java.lang.Float".equals(className))) {
			return true;
		}
		if ((nativeType == byte.class || nativeType == Byte.class)
				&& ("byte".equals(className) || "java.lang.Byte".equals(className))) {
			return true;
		}
		if ((nativeType == char.class || nativeType == Character.class)
				&& ("char".equals(className) || "java.lang.Character".equals(className))) {
			return true;
		}
		if ((nativeType == short.class || nativeType == Short.class)
				&& ("short".equals(className) || "java.lang.Short".equals(className))) {
			return true;
		}
		return false;
	}

	public static int length(Object obj) {
		if (null == obj)
			return 0;
		if (obj.getClass().isArray()) {
			return Array.getLength(obj);
		} else if (obj instanceof Collection<?>) {
			return ((Collection<?>) obj).size();
		} else if (obj instanceof Map<?, ?>) {
			return ((Map<?, ?>) obj).size();
		}
		return 1;
	}

	public static Object first(Object obj) {
		if (null == obj) {
			return null;
		}
		if (obj.getClass().isArray()) {
			if (0 == Array.getLength(obj)) {
				return null;
			}
			return Array.get(obj, 0);
		} else if (obj instanceof Collection<?>) {
			return ((Collection<?>) obj).iterator().next();
		} else if (obj instanceof Map<?, ?>) {
			return ((Map<?, ?>) obj).entrySet().iterator().next();
		}
		return obj;
	}

	public static String newUUID() {
		UUID uuid = UUID.randomUUID();
		return uuid.toString().replace("-", "");
	}

	public static ClassLoader getDefaultClassLoader() {
		ClassLoader cl = null;
		try {
			cl = Thread.currentThread().getContextClassLoader();
		} catch (Throwable ex) {
			// Cannot access thread context ClassLoader - falling back to system
			// class loader...
		}
		if (cl == null) {
			// No thread context class loader -> use class loader of this class.
			cl = BeanUtils.class.getClassLoader();
		}
		return cl;
	}

	public static boolean contains(Object[] array, Object element) {
		if (array == null) {
			return false;
		}
		for (int i = 0; i < array.length; i++) {
			if (nullSafeEquals(array[i], element)) {
				return true;
			}
		}
		return false;
	}

	public static boolean nullSafeEquals(Object o1, Object o2) {
		if (o1 == o2) {
			return true;
		}
		if (o1 == null || o2 == null) {
			return false;
		}
		if (o1.equals(o2)) {
			return true;
		}
		if (o1.getClass().isArray() && o2.getClass().isArray()) {
			if (o1 instanceof Object[] && o2 instanceof Object[]) {
				return Arrays.equals((Object[]) o1, (Object[]) o2);
			}
			if (o1 instanceof boolean[] && o2 instanceof boolean[]) {
				return Arrays.equals((boolean[]) o1, (boolean[]) o2);
			}
			if (o1 instanceof byte[] && o2 instanceof byte[]) {
				return Arrays.equals((byte[]) o1, (byte[]) o2);
			}
			if (o1 instanceof char[] && o2 instanceof char[]) {
				return Arrays.equals((char[]) o1, (char[]) o2);
			}
			if (o1 instanceof double[] && o2 instanceof double[]) {
				return Arrays.equals((double[]) o1, (double[]) o2);
			}
			if (o1 instanceof float[] && o2 instanceof float[]) {
				return Arrays.equals((float[]) o1, (float[]) o2);
			}
			if (o1 instanceof int[] && o2 instanceof int[]) {
				return Arrays.equals((int[]) o1, (int[]) o2);
			}
			if (o1 instanceof long[] && o2 instanceof long[]) {
				return Arrays.equals((long[]) o1, (long[]) o2);
			}
			if (o1 instanceof short[] && o2 instanceof short[]) {
				return Arrays.equals((short[]) o1, (short[]) o2);
			}
		}
		return false;
	}

	public static int toInt(byte[] bytes) {
		int result = 0;
		for (int i = 0; i < 4; i++) {
			result = (result << 8) - Byte.MIN_VALUE + (int) bytes[i];
		}
		return result;
	}

	public static String md5(String text) {
		return md5(text.getBytes());
	}

	/**
	 * Make MD5 diaguest.
	 */
	public static String md5(byte[] data) {
		try {
			MessageDigest md = MessageDigest.getInstance("MD5");
			byte[] buf = md.digest(data);
			return toHexString(buf);
		} catch (NoSuchAlgorithmException e) {
			throw new RuntimeException(e);
		}
	}

	private static String toHexString(byte[] bytes) {
		int length = bytes.length;
		StringBuffer sb = new StringBuffer(length * 2);
		int x = 0;
		int n1 = 0, n2 = 0;
		for (int i = 0; i < length; i++) {
			if (bytes[i] >= 0)
				x = bytes[i];
			else
				x = 256 + bytes[i];
			n1 = x >> 4;
			n2 = x & 0x0f;
			sb = sb.append(HEX[n1]);
			sb = sb.append(HEX[n2]);
		}
		return sb.toString();
	}

	/**
	 * 将一个给定的值转换为某个POJO的一个field的对应数据类型对象
	 * 
	 * @param field
	 * @param value
	 * @return
	 */
	public static Object convert(Field field, Object value) {
		if (value == null) {
			return null;
		}

		String name = null;
		try {
			name = field.getType().getName();
		} catch (Exception e) {
			throw new RuntimeException("Failed to get field's type name.", e);
		}
		return getConvertValue(name, value);
	}

	/**
	 * 将一个给定的值转换为某个POJO的一个field的对应数据类型对象
	 * 
	 * @param field
	 * @param value
	 * @return
	 */
	public static Object convert(Class<?> fieldType, Object value) {

		if (value == null) {
			return null;
		}
		String name = null;
		try {
			name = fieldType.getName();
		} catch (Exception e) {
			throw new RuntimeException("Failed to get field's type name.", e);
		}
		return getConvertValue(name, value);
	}

	public static Object getConvertValue(String name, Object value) {
		Object result = null;
		if (null == value) {
			return result;
		}
		String stringVal = value.toString();
		try {
			if (name.equals("java.lang.Byte") || name.equals("byte")) {
				try {
					result = Byte.valueOf(stringVal);
				} catch (Exception e) {
					result = 0;
				}
			} else if (name.equals("java.lang.Short") || name.equals("short")) {
				try {
					result = Short.valueOf(stringVal);
				} catch (Exception e) {
					result = 0;
				}
			} else if (name.equals("java.lang.Integer") || name.equals("int")) {
				try {
					result = Integer.valueOf(stringVal);
				} catch (Exception e) {
					result = 0;
				}
			} else if (name.equals("java.lang.Boolean") || name.equals("boolean")) {
				try {
					result = Boolean.valueOf(stringVal);
				} catch (Exception e) {
					result = false;
				}
			} else if (name.equals("java.lang.Character") || name.equals("char")) {
				try {
					result = new Character(stringVal.charAt(0));
				} catch (Exception e) {
					result = 0;
				}
			} else if (name.equals("java.lang.Double") || name.equals("double")) {
				try {
					result = Double.valueOf(stringVal);
				} catch (Exception e) {
					result = 0;
				}
			} else if (name.equals("java.lang.Float") || name.equals("float")) {
				try {
					result = Float.valueOf(stringVal);
				} catch (Exception e) {
					result = 0;
				}
			} else if (name.equals("java.lang.Long") || name.equals("long")) {
				try {
					result = Long.valueOf(stringVal);
				} catch (Exception e) {
					result = 0;
				}
			} else if (name.equals("java.lang.String")) {
				return stringVal;
//				result = String.valueOf(stringVal);
			} else if (name.equals("java.util.Date")) {
				try {
					result = DateUtils.toDate(value);
				} catch (Exception e) {
					result = null;
				}
			} else if (name.equals("java.sql.Date")) {
				try {
					result = DateUtils.toSqlDate(value);
				} catch (Exception e) {
					result = null;
				}
			} else if (name.equals("java.sql.Timestamp")) {
				try {
					result = DateUtils.toTimestamp(value);
				} catch (Exception e) {
					result = null;
				}
			} else if (name.equals("java.sql.Time")) {
				try {
					result = DateUtils.toTime(value);
				} catch (Exception e) {
					result = null;
				}
			} else if (name.equals("java.math.BigDecimal")) {
				try {
					result = new BigDecimal(stringVal.replaceAll(",", ""));
				} catch (Exception e) {
					result = null;
				}
			}
		} catch (Exception e) {
		}
		return result;
	}

	public static void copyBean(Object sourceBean, Object targetBean) {
		Class<?> classSourceType = sourceBean.getClass();
		for (java.lang.reflect.Field field : classSourceType.getDeclaredFields()) {
			copyBeanProperty(sourceBean, targetBean, field);
		}
	}

	public static void copyBeanProperty(Object sourceBean, Object targetBean, String[] property) {
		Class<?> classSourceType = sourceBean.getClass();
		List<String> propertyList = null;
		if (null != property && property.length > 0) {
			propertyList = Arrays.asList(property);
		}
		for (java.lang.reflect.Field field : classSourceType.getDeclaredFields()) {
			if (null != propertyList && propertyList.contains(field.getName())) {
				copyBeanProperty(sourceBean, targetBean, field);
			}
		}
	}

	public static void copyBeanIgnoreProperty(Object sourceBean, Object targetBean, String[] ignoreProperty) {
		List<String> ignorePropertyList = null;
		if (null != ignoreProperty && ignoreProperty.length > 0) {
			ignorePropertyList = Arrays.asList(ignoreProperty);
		}
		Class<?> classSourceType = sourceBean.getClass();
		for (java.lang.reflect.Field field : classSourceType.getDeclaredFields()) {
			if (null != ignorePropertyList && ignorePropertyList.contains(field.getName())) {
				continue;
			}
			copyBeanProperty(sourceBean, targetBean, field);
		}
	}

	private static void copyBeanProperty(Object sourceBean, Object targetBean, Field field) {
		Class<?> sourceBeanClazz = sourceBean.getClass();
		Class<?> targetBeanClazz = sourceBean.getClass();

		try {
			Method getMethod = getGetMethod(sourceBeanClazz, field);
			Method setMethod = getSetMethod(targetBeanClazz, field);
			Object value = null;
			if (null != getMethod) {
				try {
					value = getMethod.invoke(sourceBean);
				} catch (IllegalArgumentException e) {
					LOG.error(sourceBean.getClass() + "获得属性" + field.getName() + "异常", e);
				} catch (IllegalAccessException e) {
					LOG.error(sourceBean.getClass() + "获得属性" + field.getName() + "异常", e);
				} catch (InvocationTargetException e) {
					LOG.error(sourceBean.getClass() + "获得属性" + field.getName() + "异常", e);
				}

			}
			if (null != value) {
				if (isNativeType(field.getType())) {

				} else {
					try {
						if (field.getType().newInstance() instanceof BaseEntity) {
							Class<?> subBeanSourceType = field.getType().getClass();
							Object targetSubBean = field.getType().newInstance();
							for (java.lang.reflect.Field subBeanField : subBeanSourceType.getDeclaredFields()) {
								copyBeanProperty(value, targetSubBean, subBeanField);
							}
						} else if (field.getType().newInstance() instanceof Map
								|| field.getType().newInstance() instanceof List) {
							value = deepCopy(value);
						}
					} catch (InstantiationException e1) {
					} catch (IllegalAccessException e1) {
					}
				}
			}
			if (null != setMethod) {
				try {
					setMethod.invoke(targetBean, value);
				} catch (IllegalArgumentException e) {
					LOG.error("设置" + field.getName() + "属性异常", e);
				} catch (IllegalAccessException e) {
					LOG.error("设置" + field.getName() + "属性异常", e);
				} catch (InvocationTargetException e) {
					LOG.error("设置" + field.getName() + "属性异常", e);
				}
			}
		} catch (SecurityException e) {
		}

	}

	private static Object deepCopy(Object source) {
		Object targetValue = null;
		ByteArrayOutputStream bos = null;
		ObjectOutputStream oos = null;
		ObjectInputStream ois = null;
		ByteArrayInputStream bis = null;
		try {
			bos = new ByteArrayOutputStream();
			oos = new ObjectOutputStream(bos);
			oos.writeObject(source);

			// 将流序列化成对象
			bis = new ByteArrayInputStream(bos.toByteArray());

			ois = new ObjectInputStream(bis);
			targetValue = ois.readObject();
		} catch (IOException e) {
		} catch (ClassNotFoundException e) {
		} finally {
			if (null != oos) {
				try {
					oos.close();
				} catch (IOException e) {
				}
			}
			if (null != bos) {
				try {
					bos.close();
				} catch (IOException e) {
				}
			}
			if (null != bis) {
				try {
					bis.close();
				} catch (IOException e) {
				}
			}
			if (null != ois) {
				try {
					ois.close();
				} catch (IOException e) {
				}
			}
		}
		return targetValue;

	}

	public static java.lang.reflect.Method getGetMethod(Class<?> clazz, Field field) {

		Method getMethod = null;
		String property = field.getName();
		try {
			getMethod = clazz.getMethod(new StringBuilder("get").append(property.toUpperCase().substring(0, 1))
					.append(property.substring(1)).toString());
		} catch (SecurityException e) {
		} catch (NoSuchMethodException e) {
		} finally {
			if (null == getMethod) {
				if ((field.getType() == boolean.class || field.getType() == Boolean.class)
						&& property.startsWith("is")) {
					try {
						getMethod = clazz.getMethod(property);
					} catch (SecurityException e) {
					} catch (NoSuchMethodException e) {
					}
				}
			}
		}
		return getMethod;
	}

	public static java.lang.reflect.Method getSetMethod(Class<?> clazz, Field field) {
		Method setMethod = null;
		String property = field.getName();
		try {
			setMethod = clazz.getMethod(new StringBuilder("set").append(property.toUpperCase().substring(0, 1))
					.append(property.substring(1)).toString(), field.getType());
		} catch (SecurityException e) {
		} catch (NoSuchMethodException e) {
		} finally {
			if (null == setMethod) {
				if ((field.getType() == boolean.class || field.getType() == Boolean.class)
						&& property.startsWith("is")) {
					String tempProperty = property.substring(property.indexOf("is", 0) + 2);
					try {
						setMethod = clazz
								.getMethod(new StringBuilder("set").append(tempProperty.toUpperCase().substring(0, 1))
										.append(tempProperty.substring(1)).toString(), field.getType());
					} catch (SecurityException e) {
					} catch (NoSuchMethodException e) {
					}
				}
			}
		}
		return setMethod;
	}

	public static void main(String[] args) {

		System.out.println(int.class.getName() + "   " + Integer.class.getName());

		System.out.println(int.class == Integer.class);
	}

}
