package com.gx.obe.util.utils;

import java.util.Objects;

import org.apache.log4j.Logger;

public class ExceptionUtils {
	
	private static final Logger LOG = Logger.getLogger(ExceptionUtils.class);
	
	public static <T> T get(ThrowSupplier<T> s) {
		return get(null, t -> s.get(), null);
	}
	
	public static <T> T get(ThrowSupplier<T> s, T defaultValue) {
		return get(null, t -> s.get(), defaultValue);
	}
	
	public static <T, R> R get(T t, ThrowFunction<T, R> f) {
		return get(t, f, null);
	}
	
	public static <T, R> R get(T t, ThrowFunction<T, R> f, R defaultValue) {
		try {
			return f.apply(t);
		} catch (Exception e) {
			LOG.error(e.getMessage(), e);
			return defaultValue;
		}
	}
	
	public static <T, R extends AutoCloseable> void accept(T t, ThrowFunction<T, R> f, ThrowConsumer<R> c) {
		try (R r = f.apply(t)) {
			c.accept(r);
		} catch (Exception e) {
			LOG.error(e.getMessage(), e);
		}
	}
	
	public static <T extends AutoCloseable> void accept(ThrowSupplier<T> s, ThrowConsumer<T> c) {
		accept(null, t -> s.get(), c);
	}
	
	public interface ThrowSupplier<T> {
		T get() throws Exception;
	}
	
	public interface ThrowFunction<T, R> {
		R apply(T t) throws Exception;
		
		default <V> ThrowFunction<T, V> andThen(ThrowFunction<? super R, ? extends V> after) {
			Objects.requireNonNull(after);
			return (T t) -> after.apply(apply(t));
		}
	}
	
	public interface ThrowConsumer<T> {
		void accept(T t) throws Exception;
	}
	
}
