package com.gx.obe.util.utils;
import java.io.File;
import java.io.FileFilter;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

/**
 * @Description: 
 * @author guoyr
 */
public class MyClassLoader extends ClassLoader {
	
    private List<String> classPathList = new ArrayList<String>();
    private List<String> jarPathList = new ArrayList<String>();
    private Logger LOG = Logger.getLogger(MyClassLoader.class);
    
    /**
     * @Description: 动态加载类
     * @author guoyr
     */
	public MyClassLoader() {}
	
	/**
	 * @Description: 动态加载类 
	 * @author guoyr
	 * @param classPath 类存放的目录
	 * @param libPath lib库存放的目录
	 */
	public MyClassLoader(String classPath, String libPath) {
		classPathList.add(classPath);
		jarPathList.add(libPath);
	}

	/**
	 * @Description: 添加class的目录
	 * @author guoyr
	 * @param classPath
	 */
	public void addClassPath(String classPath) {
		classPathList.add(classPath);
	}
	
	/**
	 * @Description: 添加lib目录
	 * @author guoyr
	 * @param libPath
	 */
	public void addJarPath(String libPath) {
		jarPathList.add(libPath);
	}
	
	 /**
     * @Description: 动态加载类
     * @author guoyr
     * @return
     * @throws Exception
     */
    public  Class<?> findClass(String className) {
        Class<?> clazz = null;
		try {
			clazz = Class.forName(className);
		} catch (Exception e1) {
			LOG.error("加载"+className+"时错误！" , e1);
			try {
				URLClassLoader loader = new URLClassLoader(getClassAndJarURL(),this.getClass().getClassLoader());
				if(null != loader){
					clazz = loader.loadClass(className);
				}
			} catch (Exception e) {
				LOG.error("加载"+className+"时错误！", e);
				clazz = null;
			}		
		}
		return clazz;
    }
    
    /**
     * @Description: 动态加载类
     * @author guoyr
     * @return
     * @throws Exception
     */
    public  Class<?> findUrlClassFirst(String className){
    	Class<?> clazz = null;
    	
    	try {
    		URLClassLoader loader = new URLClassLoader(getClassAndJarURL(),this.getClass().getClassLoader());
    		clazz = loader.loadClass(className);
    	} catch (Exception e) {
    		LOG.error("加载"+className+"时错误！", e);
    		clazz = null;
    	}	
    	if(null == clazz){
    		LOG.error("在找定的URL下未找到类"+className+"开始从系统中加载");
    		
    		try {
        		clazz = Class.forName(className);
        	} catch (Exception e1) {
        		LOG.error("在系统中加载"+className+"时错误！" , e1);
        	}
    	}
    	return clazz;
    }
    
    /**
     * @Description: 获得类文件和lib文的URL
     * @author guoyr
     * @return
     * @throws Exception
     */
    private URL[] getClassAndJarURL(){
		List<URL> urlList = new ArrayList<URL>();

		for(String classPath : classPathList){
			File classPathFile = new File(classPath);
			if(classPathFile.exists()){
				try {
					urlList.add(classPathFile.toURL());
				} catch (MalformedURLException e) {
				}
			}
		}
		
		for(String libPath: jarPathList){
			File libPathFile = new File(libPath);
			
			if(libPathFile.exists()){
				if(libPathFile.isFile() && libPathFile.getName().endsWith(".jar")){
					try {
						urlList.add(libPathFile.toURL());
					} catch (MalformedURLException e) {
					}
				}else {
					
					File[] jarFiles = libPathFile.listFiles();
					for(File jar : jarFiles){
						if(jar.isFile() && jar.getName().endsWith(".jar")){
							try {
								urlList.add(jar.toURL());
							} catch (MalformedURLException e) {
							}
						}
					}
				}
			}
		}
    
        if(urlList.size()>0){
        	 URL[] urls = new URL[urlList.size()];
        	 for(int i=0;i<urlList.size();i++){
        		 urls[i] = urlList.get(i);
        	 }
        	return urls;
        }
        return null;
    }
    
    /**
     * @Description: jar文件过滤器
     * @author guoyr
     * @return
     * @throws Exception
     */
    public class JarFilter implements FileFilter{    
        public boolean accept(File pathname) {    
             String filename = pathname.getName().toLowerCase();    
            if(filename.contains(".jar")){    
                return false;    
             }else{    
                return true;    
             }    
         }    
     }  
    
}
