package com.gx.obe.util.utils;

import java.io.IOException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Random;
import java.util.StringTokenizer;

//import org.springframework.util.Assert;

/**
 * @Description: String 工具类
 * 
 * @version V1.0
 * @Create Date 2016-6-15 下午02:34:06 By zouk
 */
public class StringUtil {

	public static String escape(String src) {
		int i;
		char j;
		StringBuffer tmp = new StringBuffer();
		tmp.ensureCapacity(src.length() * 6);

		for (i = 0; i < src.length(); i++) {

			j = src.charAt(i);

			if (Character.isDigit(j) || Character.isLowerCase(j)
					|| Character.isUpperCase(j))
				tmp.append(j);
			else if (j < 256) {
				tmp.append("%");
				if (j < 16)
					tmp.append("0");
				tmp.append(Integer.toString(j, 16));
			} else {
				tmp.append("%u");
				tmp.append(Integer.toString(j, 16));
			}
		}
		return tmp.toString();
	}

	/**
	 * @param src
	 * @return
	 *
	 * @version:v1.0
	 * @author:zouk
	 * @date:2016年6月18日 下午2:11:43
	 */
	public static String unescape(String src) {
		StringBuffer tmp = new StringBuffer();
		tmp.ensureCapacity(src.length());
		int lastPos = 0, pos = 0;
		char ch;
		while (lastPos < src.length()) {
			pos = src.indexOf("%", lastPos);
			if (pos == lastPos) {
				if (src.charAt(pos + 1) == 'u') {
					ch = (char) Integer.parseInt(src
							.substring(pos + 2, pos + 6), 16);
					tmp.append(ch);
					lastPos = pos + 6;
				} else {
					ch = (char) Integer.parseInt(src
							.substring(pos + 1, pos + 3), 16);
					tmp.append(ch);
					lastPos = pos + 3;
				}
			} else {
				if (pos == -1) {
					tmp.append(src.substring(lastPos));
					lastPos = src.length();
				} else {
					tmp.append(src.substring(lastPos, pos));
					lastPos = pos;
				}
			}
		}
		return tmp.toString();
	}

 
	/**用Base64算法对字符串进行编码
	 * @param str
	 * @return
	 *
	 * @version:v1.0
	 * @author:zouk
	 * @date:2016年6月18日 下午2:11:47
	 */
	public static String encodeString(String str) {
		sun.misc.BASE64Encoder encoder = new sun.misc.BASE64Encoder();
		return encoder.encodeBuffer(str.getBytes()).trim();
	}

	 
	/**用Base64算法对字符串进行解码
	 * @param str
	 * @return
	 *
	 * @version:v1.0
	 * @author:zouk
	 * @date:2016年6月18日 下午2:11:55
	 */
	public static String decodeString(String str) {
		sun.misc.BASE64Decoder dec = new sun.misc.BASE64Decoder();
		try {
			return new String(dec.decodeBuffer(str));
		} catch (IOException io) {
			throw new RuntimeException(io.getMessage(), io.getCause());
		}
	}

 
	/**判断对象是否为空,或者为长度
	 * @param obj
	 * @return
	 *
	 * @version:v1.0
	 * @author:zouk
	 * @date:2016年6月18日 下午2:12:03
	 */
	public static boolean empty(Object obj) {
		boolean result = true;
		if (null == obj) {
			result = true;
		} else if (obj instanceof String) {
			result = ((String) obj).length() == 0 ? true : false;
		} else if (obj instanceof List) {
			result = ((List<?>) obj).size() == 0 ? true : false;
		} else if (obj instanceof String[]) {
			result = ((String[]) obj).length == 0 ? true : false;
		}
		return result;
	}

	public static String[] toArray(String parseString) {
		return toArray(parseString, " \t\n\r\f", false);
	}

	public static String[] toArray(String parseString, String splitString) {
		return toArray(parseString, splitString, false);
	}

	/**
	 * 分隔一个字符串
	 * 
	 * @param parseString
	 *            String 原始字符串
	 * @param splitString
	 *            String 分隔串
	 * @param returnDelims
	 *            boolean 返回值是否包含分隔串
	 * @return String[] 分隔后的字符串
	 */
	public static String[] toArray(String parseString, String splitString,
			boolean returnDelims) {
		StringTokenizer tokens = new StringTokenizer(parseString, splitString,
				returnDelims);
		String[] values = new String[tokens.countTokens()];
		int i = 0;
		while (tokens.hasMoreTokens()) {
			values[i++] = tokens.nextToken();
		}
		return values;
	}

	 
	/**两个字段串相除,并保存6位小
	 * @param objOne
	 * @param objTwo
	 * @return
	 *
	 * @version:v1.0
	 * @author:zouk
	 * @date:2016年6月18日 下午2:12:34
	 */
	public static String divide(String objOne, String objTwo) {
		return divide(objOne, objTwo, 6);
	}

	 
	/**两字段串相除,保留小数位数可
	 * @param objOne
	 * @param objTwo
	 * @param scale
	 * @return
	 *
	 * @version:v1.0
	 * @author:zouk
	 * @date:2016年6月18日 下午2:12:46
	 */
	public static String divide(String objOne, String objTwo, int scale) {

		BigDecimal one = new BigDecimal(objOne);
		BigDecimal two = new BigDecimal(objTwo);
		return one.divide(two, scale, RoundingMode.HALF_UP).toString();

	}

	 
	/**清除字段串有点的数据最后带字符
	 * @param inpuStr
	 * @return
	 *
	 * @version:v1.0
	 * @author:zouk
	 * @date:2016年6月18日 下午2:12:54
	 */
	public static String clearNumZero(String inpuStr) {
		String pointEnd = inpuStr.substring(inpuStr.lastIndexOf(".") + 1);
		char[] c = pointEnd.toCharArray();
		int ind = inpuStr.indexOf(".");
		if (ind == -1)
			return inpuStr;
		StringBuffer str = new StringBuffer();
		str.append(inpuStr.substring(0, ind));
		int n = 0;
		for (int i = c.length - 1; i > 0; i--) {
			if (c[i] != '0' && c[i] != '.') {
				n = i;
				break;
			}
		} 
		if (n == 0 && c[0] == '0') {
		} else {
			str.append(".");
			for (int i = 0; i <= n; i++) {
				str.append(c[i]);
			}
		}
		return str.toString();
	}

	
	
	 
	/**生成随机数
	 * @param length
	 * @return
	 *
	 * @version:v1.0
	 * @author:zouk
	 * @date:2016年6月18日 下午2:13:06
	 */
	public static final String randomString(int length) {
		return randomString("0123456789abcdefghijklmnopqrstuvwxyz", length);
	}
	
	 
	/**从某个字段中随机生成几位字符串
	 * @param str
	 * @param length
	 * @return
	 *
	 * @version:v1.0
	 * @author:zouk
	 * @date:2016年6月18日 下午2:13:13
	 */
	public static final String randomString(String str,int length) {
		if (length < 1) {
			return null;
		}
		if(str==null||str.length()==0){
			return null;
		}
		Random randGen = new Random();
		char[] numbersAndLetters =str.toCharArray();
		char[] randBuffer = new char[length];
		for (int i = 0; i < randBuffer.length; i++) {
			randBuffer[i] = numbersAndLetters[randGen.nextInt(35)];
		}
		return new String(randBuffer);//generatorSequence
	}
	
	public static char toUpperCase(char ch) {
        if (ch >= 'a' && ch <= 'z') {
            ch -= 32;
        }
        return ch;
    }

   
    /**将给定字符串指定位置上的字母大写
     * @param source
     * @param pos
     * @return
     *
     * @version:v1.0
     * @author:zouk
     * @date:2016年6月18日 下午2:13:22
     */
    public static String upperCharAt(final String source, int pos) {
        if (source == null) {
            return null;
        }
        int len = source.length();
        if (pos < 0 || pos >= len) {
            return "";
        }
        char[] chars = source.toCharArray();
        chars[pos] = toUpperCase(chars[pos]);
        return new String(chars);
    }
    
    
    /**将给定字符串所有字母大写
     * @param source
     * @return
     *
     * @version:v1.0
     * @author:zouk
     * @date:2016年6月18日 下午2:13:29
     */
    public static String upperStrintAt(final String source) {
        if (source == null) {
            return null;
        }
        char[] chars = source.toCharArray();
        StringBuffer returnString = new StringBuffer();
        for (int i = 0; i < chars.length; i++) {
        	chars[i] = toUpperCase(chars[i]);
		}
        return new String(chars);
    }
    	
   
    public static void main(String[] args) {
    	System.out.println(StringUtil.encodeString("linjin--3826"));
	}
}
