package com.gx.obe.util.utils;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

/**
 * @Description: 
 * @author guoyr
 */
public class XmlResolver {

	private Document document = null;
	private Element rootDocument = null;
	private Logger LOG = Logger.getLogger(XmlResolver.class);
	private File xmlFile = null;
	public XmlResolver(String xmlPath){
		xmlFile = new File(xmlPath);
		init();
	}
	public XmlResolver(File xmlFile){
		this.xmlFile = xmlFile;
		init();
	}
	
	private void init(){
		
		SAXReader reader = new SAXReader();
		try {
			if(null != xmlFile && xmlFile.exists()){
				document = reader.read(xmlFile);// 读取XML文档
				rootDocument = document.getRootElement();// 声明跟元素
			}
		} catch (DocumentException e) {
			LOG.error("读取"+xmlFile+"文件时出错", e);
		}
	}
	
	public Element getRootElement(){
		return rootDocument;
	}
	
	public Element getElement(Element parentElement, String elementNames){
		if(null == parentElement){
			parentElement = rootDocument;
		}
		if(null == parentElement){
			return null;
		}
		Element element = null;
		String[] elementNameArray = elementNames.split("/");
		if(elementNameArray.length > 1){
			String elementName = elementNameArray[0];
//			element = parentElement.element(elementName);
			element = getElementFilterAttr(parentElement,elementName);
			for(int i = 1; i < elementNameArray.length; i ++){
//				element = element.element(elementNameArray[i]);
				element = getElementFilterAttr(element, elementNameArray[i]);
				if(null == element){
					LOG.error(parentElement.getName()+"下不存在"+elementNameArray[i]+"节点");
					break;
				}
			}
		}else {
//			element = (Element) parentElement.element(elementNames);
			element = getElementFilterAttr(parentElement, elementNames);;
		}
		return element;
		
	}
	public List<Element> getElementList(Element parentElement, String elementNames){
		if(null == parentElement){
			parentElement = rootDocument;
		}
		if(null == parentElement){
			return null;
		}
		List<Element> elementList = null;
		String[] elementNameArray = elementNames.split("/");
		
		if(elementNameArray.length > 1){
//			Element element = parentElement.element(elementNameArray[0]);
			Element element = getElementFilterAttr(parentElement,elementNameArray[0]);
			for(int i = 1; i < elementNameArray.length - 1; i ++){
//				element = element.element(elementNameArray[i]);
				element = getElementFilterAttr(element, elementNameArray[i]);
				if(null == element){
					LOG.error(parentElement.getName()+"下不存在"+elementNameArray[i]+"节点");
					break;
				}
			}
			if(null != element){
//				elementList = (List<Element>)element.elements(elementNameArray[elementNameArray.length -1]);
				elementList = getElementListFilterAttr(element, elementNameArray[elementNameArray.length -1]);
			}
		
		}else {
//			elementList = (List<Element>)parentElement.elements(elementNames);
			elementList = getElementListFilterAttr(parentElement,elementNames);
		}
		return elementList;
	}
	
	private Element getElementFilterAttr(Element parentElement, String elementName){
		if(null == parentElement){
			return null;
		}
		List<Element> elementList = getElementListFilterAttr(parentElement, elementName);
		if(null != elementList && elementList.size() > 0){
			if(elementList.size() > 1){
				LOG.error(parentElement.getName()+"下存在多个"+elementName+"节点");
			}
			return elementList.get(0);
		}else {
			return null;
		}
	}
	@SuppressWarnings("unchecked")
	private List<Element> getElementListFilterAttr(Element parentElement, String elementName){
		if(null == parentElement){
			return null;
		}
		List<Element> elementList = new ArrayList<Element>();
		Element element = null;
		if(elementName.indexOf("[") > 0 && elementName.indexOf("]") == elementName.length() -1){
			int firstPoint = elementName.indexOf("[", 0);
			String attributes = elementName.substring(firstPoint, elementName.length() -1);
			attributes = attributes.replace("[", "");
			String[] attributeArray = attributes.split("]");
			elementName = elementName.substring(0, firstPoint);
			List<Element> _elementList = (List<Element>)parentElement.elements(elementName);
			if(null != _elementList && _elementList.size() > 0){
				for(Element e : _elementList){
					boolean flag = true;
						for(String attribute : attributeArray){
							String atValue = e.attributeValue(attribute.split("=")[0]);
							if(null == atValue || !atValue.equals(attribute.split("=")[1])){
								flag = false;
								break;
							}
						}
						if(flag){
							elementList.add(e);
						}
				}
			}else {
				return null;
			}
		}else {
			elementList.addAll(parentElement.elements(elementName));
		}
		return elementList;
	}
}
