package com.gx.obe.util.utils;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;

/**
 * @Description:
 * @author guoyr
 */
public class XmlUtils {
	private static Logger LOG = Logger.getLogger(XmlUtils.class);
	
	/**
	 * @Description: 修改XML文件
	 * @author chenxw
	 * @param xmlFile
	 * @param updateXml
	 */
	public static void updateXmlFile(File xmlFile, UpdateXml updateXml) {
		Document document = getXmlDocument(xmlFile.getAbsolutePath());
		updateXml.update(document.getRootElement());
		createXmlFile(document, xmlFile.getAbsolutePath());
	}
	
	public interface UpdateXml {
		void update(Element rootElement);
	}
	
	/**
	 * @Description: 获取xml的Document
	 * @author guoyr
	 * @param xmlPath
	 * @return
	 * @throws Exception
	 */
	public static Document getXmlDocument(String xmlPath) {
		Document doc = null;// 声明文档对象
		
		SAXReader reader = new SAXReader();
		try {
			File xmlFile = new File(xmlPath);
			if (null != xmlFile && xmlFile.exists()) {
				doc = reader.read(xmlFile);// 读取XML文档
			}
		} catch (DocumentException e) {
			LOG.error("读取" + xmlPath + "文件时出错", e);
			
			FileInputStream in = null;
			Reader read = null;
			// xml读取异常Invalid byte 1 of 1-byte UTF-8 sequence
			try {
				in = new FileInputStream(new File(xmlPath));
				read = new InputStreamReader(in, "gbk");
				doc = reader.read(read);
			} catch (FileNotFoundException e1) {
				LOG.error(e1);
			} catch (UnsupportedEncodingException e1) {
				LOG.error(e1);
			} catch (DocumentException e1) {
				LOG.error(e1);
			} finally {
				if (null != in) {
					try {
						in.close();
						in = null;
					} catch (IOException e1) {}
				}
				if (null != read) {
					try {
						read.close();
						read = null;
					} catch (IOException e1) {}
				}
			}
		}
		return doc;
	}
	
	/**
	 * @Description: 创建xml文件
	 * @author guoyr
	 * @param xmlName 要创建的xml文件路径
	 * @param xmlStr 要创建的xml内容
	 * @param encoding 指定XML字符集编码
	 * @return
	 * @Exception
	 */
	public static boolean createXmlFile(String xmlName, String xmlStr, String... encoding) {
		Document document;
		boolean creatSuccess = false;
		try {
			document = DocumentHelper.parseText(xmlStr);
			createXmlFile(document, xmlName, null != encoding && encoding.length > 0 ? encoding[0] : null);
			creatSuccess = true;
		} catch (DocumentException e) {
			LOG.error("创建" + xmlName + "文件时出错", e);
		}
		return creatSuccess;
	}
	
	/**
	 * @Description:
	 * @author guoyr
	 * @param document
	 * @param xmlPath xml文件全路径
	 * @param encoding 指定XML字符集编码
	 * @return
	 */
	public static boolean createXmlFile(Document document, String xmlPath, String... encoding) {
		XMLWriter writer = null;
		FileOutputStream fileOutputStream = null;
		OutputStreamWriter outputStreamWriter = null;
		try {
			if (!new File(xmlPath).getParentFile().exists()) {
				new File(xmlPath).getParentFile().mkdirs();
			}
			
			/** 格式化输出,类型IE浏览一样 */
			OutputFormat format = OutputFormat.createPrettyPrint();
//			OutputFormat format = OutputFormat.createCompactFormat();
			
			String _encoding = "UTF-8";
			/** 指定XML字符集编码 */
			if (null != encoding && encoding.length > 0) {
				_encoding = encoding[0];
			}
			document.setXMLEncoding(_encoding);
			format.setEncoding(_encoding);
			fileOutputStream = new FileOutputStream(xmlPath);
			outputStreamWriter = new OutputStreamWriter(fileOutputStream, _encoding);
			
			//换成下面的方法，才会生成UTF-8格式的xml文件，不然读的时候会报错    org.dom4j.DocumentException:   Invalid   byte   1   of   1-byte   UTF-8   sequence.  
			writer = new XMLWriter(outputStreamWriter, format);
			writer.setEscapeText(false);
			writer.write(document);
			writer.flush();
			
			return true;
		} catch (Exception ex) {
			return false;
		} finally {
			if (null != fileOutputStream) {
				try {
					fileOutputStream.close();
					fileOutputStream = null;
				} catch (IOException e) {}
			}
			if (null != outputStreamWriter) {
				try {
					outputStreamWriter.close();
					outputStreamWriter = null;
				} catch (IOException e) {}
			}
			if (null != writer) {
				try {
					writer.close();
					writer = null;
				} catch (IOException e) {}
			}
		}
		
	}
	
	public static Document getXmlDocument(byte[] documentByte) {
		if (null == documentByte) {
			return null;
		}
		Document document = getXmlDocument(documentByte, "UTF-8");
		if (null == document) {
			document = getXmlDocument(documentByte, "gbk");
		}
		return document;
	}
	
	public static Document getXmlDocument(byte[] documentByte, String encoding) {
		
		if (null == documentByte) {
			return null;
		}
		Document doc = null;
		ByteArrayInputStream in = null;
		SAXReader reader = new SAXReader();
		Reader read = null;
		// xml读取异常Invalid byte 1 of 1-byte UTF-8 sequence
		try {
			in = new ByteArrayInputStream(documentByte);
			read = new InputStreamReader(in, "UTF-8");
			doc = reader.read(read);
		} catch (UnsupportedEncodingException e1) {
			LOG.error(e1);
		} catch (DocumentException e1) {
			LOG.error(e1);
		} finally {
			if (null != in) {
				try {
					in.close();
					in = null;
				} catch (IOException e1) {}
			}
			if (null != read) {
				try {
					read.close();
					read = null;
				} catch (IOException e1) {}
			}
		}
		return doc;
		
	}
	
	public static byte[] getDocumentByte(Document document) {
		
		byte[] byteArr = null;
		if (null != document) {
			XMLWriter xmlWriter = null;
			ByteArrayOutputStream byteOut = null;
			try {
				byteOut = new ByteArrayOutputStream();
				xmlWriter = new XMLWriter(byteOut);
				xmlWriter.write(document);
				byteArr = byteOut.toByteArray();
			} catch (UnsupportedEncodingException e) {} catch (IOException e) {} finally {
				if (null != xmlWriter) {
					try {
						xmlWriter.close();
					} catch (IOException e) {}
				}
				if (null != byteOut) {
					try {
						byteOut.close();
					} catch (IOException e) {}
				}
				if (null == byteArr) {
					String tempPath = System.getProperty("java.io.tmpdir") + new Date().getTime() + ".xml";
					XmlUtils.createXmlFile(document, tempPath, "UTF-8");
				}
			}
		}
		return byteArr;
	}
	
	public static String getXmlText(Date value, String format) {
		if (null != value && null != format) {
			return getXmlText(value, new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"));
		}
		return "";
	}
	
	public static String getXmlText(Date value, SimpleDateFormat format) {
		if (null != value) {
			return getXmlText(null != format ? format.format(value) : value.toString());
		}
		return "";
	}
	
	public static String getXmlText(Object value) {
		if (null != value) {
			return getXmlText(value.toString());
		}
		return "";
	}
	
	public static String getXmlText(String value) {
		if (null != value && value.length() > 0) {
			return "<![CDATA[" + value + "]]>";
//			return value;
		}
		return "";
	}
	
	public static boolean updateElementText(Element parentElement, String elementName, String text){
		Element element = parentElement.element(elementName);
		if(null == element){
			element = parentElement.addElement(elementName);
		}
		if(null != element){
			element.setText(getXmlText(text));
			return true;
		}
		return false;
	}
	
	public static Element getOrCreateElement(Element parentElement, String elementName){
		Element element = parentElement.element(elementName);
		if(null == element){
			element = parentElement.addElement(elementName);
		}
		return element;
	}
	
	public static String getXmlString(Document document) {
		OutputFormat outputFormat = OutputFormat.createPrettyPrint();
		outputFormat.setEncoding("UTF-8");
		outputFormat.setSuppressDeclaration(true);
		outputFormat.setIndent(true);
		outputFormat.setIndent("    ");
		outputFormat.setNewlines(true);
		StringWriter stringWriter = new StringWriter();
		XMLWriter xmlWriter = new XMLWriter(stringWriter, outputFormat);
		try {
			xmlWriter.write(document);
		} catch (IOException e) {}
		try {
			xmlWriter.close();
		} catch (IOException e) {}
		return stringWriter.toString();
	}
	
}
